@@ -164,6 +164,7 @@ protected:
     , mIsRepeat(false)
     , mIsComposing(false)
     , mIsSynthesizedByTIP(false)
+    , mMaybeSkippableInRemoteProcess(true)
     , mEditCommandsForSingleLineEditorInitialized(false)
     , mEditCommandsForMultiLineEditorInitialized(false)
     , mEditCommandsForRichTextEditorInitialized(false)
@@ -192,6 +193,7 @@ public:
     , mIsRepeat(false)
     , mIsComposing(false)
     , mIsSynthesizedByTIP(false)
+    , mMaybeSkippableInRemoteProcess(true)
     , mEditCommandsForSingleLineEditorInitialized(false)
     , mEditCommandsForMultiLineEditorInitialized(false)
     , mEditCommandsForRichTextEditorInitialized(false)
@@ -348,6 +350,25 @@ public:
   // Indicates whether the event is synthesized from Text Input Processor
   // or an actual event from nsAppShell.
   bool mIsSynthesizedByTIP;
+  // Indicates whether the event is skippable in remote process.
+  // Don't refer this member directly when you need to check this.
+  // Use CanSkipInRemoteProcess() instead.
+  bool mMaybeSkippableInRemoteProcess;
+
+  bool CanSkipInRemoteProcess() const
+  {
+    // If this is a repeat event (i.e., generated by auto-repeat feature of
+    // the platform), remove process may skip to handle it because of
+    // performances reasons..  However, if it's caused by odd keyboard utils,
+    // we should not ignore any key events even marked as repeated since
+    // generated key sequence may be important to input proper text.  E.g.,
+    // "SinhalaTamil IME" on Windows emulates dead key like input with
+    // generating WM_KEYDOWN for VK_PACKET (inputting any Unicode characters
+    // without keyboard layout information) and VK_BACK (Backspace) to remove
+    // previous character(s) and those messages may be marked as "repeat" by
+    // their bug.
+    return mIsRepeat && mMaybeSkippableInRemoteProcess;
+  }
 
   /**
    * Retrieves all edit commands from mWidget.  This shouldn't be called when
@@ -576,6 +597,7 @@ public:
     mPluginTextEventString.Assign(aEvent.mPluginTextEventString);
 #endif
     mIsSynthesizedByTIP = aEvent.mIsSynthesizedByTIP;
+    mMaybeSkippableInRemoteProcess = aEvent.mMaybeSkippableInRemoteProcess;
 
     // Don't copy mEditCommandsFor*Editor because it may require a lot of
     // memory space.  For example, if the event is dispatched but grabbed by
