@@ -726,6 +726,9 @@ CreateHeaderBarButtons()
 
   GtkWidget *buttonBox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, buttonSpacing);
   gtk_container_add(GTK_CONTAINER(GetWidget(MOZ_GTK_HEADER_BAR)), buttonBox);
+  // We support only LTR headerbar layout for now.
+  gtk_style_context_add_class(gtk_widget_get_style_context(buttonBox),
+                              GTK_STYLE_CLASS_LEFT);
 
   WidgetNodeType buttonLayout[TOOLBAR_BUTTONS];
   int activeButtons =
